﻿/**
 * This is a part of MCServe, which is available under the Apache license 2.0.
 * With this code, you may do whatever you want as long as that is not in conflict with this license.
 * In addition, you should have a reference to the original project to include (http://mcserve.codeplex.com).
 * 
 * Author       : Lars Krämer ( Xe77a )
 * Website      : http://www.lkraemer.de
 * Project      : MCServe (http://mcserve.codeplex.com)
 * Copyright    : Copyright (c) 2013 lkraemer.de
 * License      : http://mcserve.codeplex.com/license
 * Description  : This is a part of the core functionality of the server
 */

using MCServeLib.Types;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;

namespace MCServeCore.Components
{
    public class ServerIpList
    {
        private ObservableCollection<IpObject> _Ips = new ObservableCollection<IpObject>();
        public ObservableCollection<IpObject> Ips
        {
            get
            {
                return _Ips;
            }
        }

        /**
         * Konstruktor
         */
        public ServerIpList()
        {
            Load();
        }

        public void Add(IpObject ip)
        {
            _Ips.Add(ip);
            Save();
        }

        public void Remove(IpObject ipcode)
        {
            for (int i = _Ips.Count - 1; i >= 0; i--)
            {
                if (_Ips[i].Ip == ipcode.Ip)
                {
                    _Ips.RemoveAt(i);
                    Save();
                }
            }
        }

        private Boolean Load()
        {
            if (File.Exists(GameServer.Paths.Get(MCServePaths.LIST_BANNED_IPS)))
            {
                StreamReader ConfigReader = new StreamReader(GameServer.Paths.Get(MCServePaths.LIST_BANNED_IPS));

                List<String> List = new List<String>();
                List = JsonConvert.DeserializeObject<List<String>>(ConfigReader.ReadToEnd());

                if (List == null)
                {
                    return true;
                }

                foreach (String IP in List)
                {
                    IpObject ip = new IpObject();
                    ip.Ip = IP;
                    Add(ip);
                }

                ConfigReader.Close();

                return true;
            }
            else
            {
                return false;
            }
        }

        private Boolean Save()
        {
            if (GameServer.Paths.Get(MCServePaths.LIST_BANNED_IPS) == null)
            {
                return false;
            }

            try
            {
                File.Delete(GameServer.Paths.Get(MCServePaths.LIST_BANNED_IPS));

                List<String> List = new List<String>();

                foreach (IpObject ip in Ips)
                {
                    List.Add(ip.Ip);
                }

                StreamWriter ConfigWriter = new StreamWriter(GameServer.Paths.Get(MCServePaths.LIST_BANNED_IPS));
                ConfigWriter.Write(JsonConvert.SerializeObject(List));
                ConfigWriter.Close();
                return true;
            }
            catch (Exception err)
            {
                GameServer.Console.SendMessage(err.Message, ServerConsole.OutputTarget.ErrorLogAndConsole, ServerConsole.OutputTypes.Error);
                return false;
            }
        }
    }
}
